package cn.estsh.i3plus.ext.mes.pcn.apiservice.controller.busi;


import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesInputDefectRecordService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.pojo.mes.bean.MesDefectRecord;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@RestController
@Api(tags = "缺陷记录页面")
@RequestMapping(MesCommonConstant.MES_YANFEN + "/defect_record")
@Slf4j
public class MesInputDefectRecordController {

    @Autowired
    private IMesInputDefectRecordService inputDefectRecordService;


    @GetMapping("/query-loading-order")
    @ApiOperation(value = "查询产品条码")
    public ResultBean queryProduceSn(String serialNumber, Integer frontBack) {

        try {
            // 数据校验
            ValidatorBean.checkNotNull(serialNumber, "产品条码不能为空");
            ValidatorBean.checkNotNull(frontBack, "正反面不能为空");

            return ResultBean.success("查询成功").setResultObject(inputDefectRecordService.queryProduceSn(serialNumber, frontBack, AuthUtil.getOrganizeCode()));
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("/query-part-lot")
    @ApiOperation(value = "查询零件号-批次")
    public ResultBean queryProduceSn(String partNo, String lot, Integer frontBack) {

        try {
            // 数据校验
            ValidatorBean.checkNotNull(partNo, "物料不能为空");
            ValidatorBean.checkNotNull(lot, "批次不能为空");
            ValidatorBean.checkNotNull(frontBack, "正反面不能为空");

            return ResultBean.success("查询成功").setResultObject(inputDefectRecordService.queryPartAndLot(partNo, lot, frontBack, AuthUtil.getOrganizeCode()));
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("/query-part-type-defect")
    @ApiOperation(value = "查询零件类型缺陷类型对应关系")
    public ResultBean queryPartTypeDefect(String partNo, String defectLocation, boolean flg, Integer frontBack) {

        try {
            // 数据校验
            ValidatorBean.checkNotNull(partNo, "物料号不能为空");
            ValidatorBean.checkNotNull(defectLocation, "位置不能为空");
            ValidatorBean.checkNotNull(frontBack, "正反面不能为空");

//            return ResultBean.success("查询成功").setResultList(inputDefectRecordService.queryPartTypeDefect(partNo, defectLocationCode, flg, frontBack, "2031"));
            return ResultBean.success("查询成功").setResultList(inputDefectRecordService.queryPartTypeDefect(partNo, defectLocation, flg, frontBack, AuthUtil.getOrganizeCode()));
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping("/save-defect-record")
    @ApiOperation(value = "完成质检-保存不良记录")
    public ResultBean saveDefectRecord(@RequestBody List<MesDefectRecord> defectRecordList) {

        try {
            // 数据校验
            ValidatorBean.checkNotNull(defectRecordList, "选择不良数据不能为空");
            inputDefectRecordService.saveDefectRecord(defectRecordList, AuthUtil.getOrganizeCode());
            return ResultBean.success("质检已完成，请继续");
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


}
