package cn.estsh.i3plus.ext.mes.pcn.apiservice.controller.busi;


import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesNcProcessingService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesNcProcessingInputModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesNcProcessingPartAndLotModel;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;

@RestController
@Api(tags = "NC处理页面")
@RequestMapping(MesCommonConstant.MES_YANFEN + "/nc-processing")
@Slf4j
public class MesNcProcessingController {

    @Autowired
    private IMesNcProcessingService ncProcessingService;


    @GetMapping("/query")
    @ApiOperation(value = "查询产品条码或零件号+批次")
    public ResultBean queryProduceSn(String serialNumber, String partNo, String lot) {

        try {
            // 数据校验
            if (StringUtils.isEmpty(serialNumber) && (StringUtils.isEmpty(partNo) || StringUtils.isEmpty(lot))) {
                throw new ImppBusiException("产品条码为空或零件号+批次为空");
            }

            return ResultBean.success("查询成功").setResultObject(ncProcessingService.queryProduceSn(serialNumber, partNo, lot, AuthUtil.getOrganizeCode()));
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping("/query-part-lot")
    @ApiOperation(value = "查询零件批次")
    public ResultBean queryPartLot(String partNo, String lot, Pager pager) {

        try {
            // 数据校验
//            ValidatorBean.checkNotNull(partNo, "零件号不能为空");
//            ValidatorBean.checkNotNull(lot, "不能为空");

            ListPager<MesNcProcessingPartAndLotModel> modelList = ncProcessingService.queryPartLot(partNo, lot, AuthUtil.getOrganizeCode(), pager);
            return ResultBean.success("查询成功").setListPager(modelList);
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping("/save-nc")
    @ApiOperation(value = "NC处理")
    public ResultBean saveNc(@RequestBody MesNcProcessingInputModel model) {

        try {
            // 数据校验
            ValidatorBean.checkNotNull(model.getType(), "类型不能为空");
            if (StringUtils.isEmpty(model.getSerialNumber()) && (StringUtils.isEmpty(model.getPartNo()) || StringUtils.isEmpty(model.getLot()))) {
                throw new ImppBusiException("产品条码为空或零件号+批次为空");
            }

            String description = MesExtEnumUtil.NC_TYPE.valueOfDescription(model.getType());
            if (StringUtils.isEmpty(description)) {
                throw new ImppBusiException(String.format("【%s】类型不存在，请检查数据", model.getType()));

            }

            ncProcessingService.saveNc(model, AuthUtil.getOrganizeCode());
            return ResultBean.success("NC处理成功");
        } catch (ImppBusiException imppException) {
            return ResultBean.fail(imppException);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

}
