package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesSnProductionProcessContext;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesMouldMultiCavity;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCell;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Description : 生成零件条码
 * @Author : wangjie
 **/
@Slf4j
@Service
public class MesProductSnGenerateStepService extends BaseStepService {
    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private MesPartRepository mesPartRepository;

    @Autowired
    private MesProduceSnRepository mesProduceSnRepository;

    @Autowired
    private ISyncFuncService syncFuncService;

    public static final String ORDER_NO_JIS_SORT = "ORDER_NO_JIS_SORT";


    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        StepResult stepResult = StepResult.getSuccessComplete();

        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.getEquipmentVariableList(reqBean, MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PRODUCTION.getValue());
        MesWorkCell mesWorkCell = productionProcessContext.getWorkCell();
        List<MesMouldMultiCavity> mesModelMultiCavities = productionProcessContext.getMouldCavityList();
        List<MesProduceSn> mesProduceSns = new ArrayList<>();
        for (MesMouldMultiCavity modelMultiCavity : mesModelMultiCavities) {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(reqBean.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(modelMultiCavity.getPartNo(), "partNo", packBean);
            MesPart mesPart = mesPartRepository.getByProperty(packBean);
            if (mesPart == null) {
                return execNonCompleteAndSendMsgReturn(reqBean, resultBean, stepResult, "零件号不存在");
            }
            String sn = generateByRule(mesPart);
            MesProduceSn mesProduceSn = generateMesProduceSn(mesPart, sn,mesWorkCell, productionProcessContext,reqBean);
            mesProduceSns.add(mesProduceSn);
        }
        mesProduceSnRepository.saveAll(mesProduceSns);
        MesSnProductionProcessContext mesSnProductionProcessContext = (MesSnProductionProcessContext) productionProcessContext;
        mesSnProductionProcessContext.productSns(mesProduceSns);

        return StepResult.getSuccessComplete();
    }


    private String generateByRule(MesPart mesPart) {
        //生成工单号
        GenSerialNoModel genSerialNoModel = new GenSerialNoModel(ORDER_NO_JIS_SORT);
        genSerialNoModel.setPartNo(mesPart.getPartNo());
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel, mesPart.getCreateUser(), mesPart.getOrganizeCode(), 1).getResultList();
        return  resultList.get(0);
    }

    private MesProduceSn generateMesProduceSn(MesPart mesPart, String sn, MesWorkCell mesWorkCell,MesProductionProcessContext productionProcessContext, StationRequestBean reqBean) {
        long serialNum = snowflakeIdMaker.nextId();

        MesProduceSn mesProduceSn = new MesProduceSn();
        mesProduceSn.setSerialNumber(serialNum + "");
        mesProduceSn.setProductSn(sn);
        mesProduceSn.setCustSn(sn);
        mesProduceSn.setPartNo(mesPart.getPartNo());
        mesProduceSn.setPartNameRdd(mesPart.getPartName());
        mesProduceSn.setAreaCode(mesWorkCell.getAreaCode());
        mesProduceSn.setWorkCenterCode(mesWorkCell.getWorkCenterCode());
        mesProduceSn.setWorkCellCode(mesWorkCell.getWorkCellCode());
        mesProduceSn.setProcessLabelTemplate(mesPart.getProcessLabelTemplate());
        mesProduceSn.setCustLabelTemplate(mesPart.getCustLabelTemplate());
        mesProduceSn.setProcessLabelTemplate(mesPart.getProcessLabelTemplate());
        mesProduceSn.setQty(1d);
        mesProduceSn.setSnStatus(10);
        mesProduceSn.setQcStatus(MesExtEnumUtil.PRODUCE_QC_STATUS.QUALIFIED.getValue());
        mesProduceSn.setLotNo(DateUtil.format(new Date(), "YYYYMMDD"));
        mesProduceSn.setRouteCode(reqBean.getRouteCode());
        mesProduceSn.setProcessCode(productionProcessContext.getProcessCode());
        mesProduceSn.setCraftCode(productionProcessContext.getCraftCode());
        mesProduceSn.setPrintCount(0);
        mesProduceSn.setPrintStatus(10);

        return  mesProduceSn;
    }
}
