package cn.estsh.i3plus.ext.mes.pcn.pojo.context;

import cn.estsh.i3plus.pojo.mes.bean.*;
import com.alibaba.fastjson.JSONObject;
import io.swagger.annotations.ApiParam;
import lombok.Data;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.TreeSet;
import java.util.stream.Collectors;

/**
 * 生产过程上下文对象
 */
@Data
public class MesProductionProcessContext {

    @ApiParam("结果")
    private Boolean success;

    @ApiParam("信息")
    private String message;

    @ApiParam("组织代码")
    private String organizeCode;

    @ApiParam("生产线信息([Json]MesWorkCenter)")
    private String workCenterJson;

    @ApiParam("工位信息([Json]MesWorkCell)")
    private String workCellJson;

    @ApiParam("工位设备信息([Json]List<MesCellEquipContext>)")
    private String cellEquipListJson;

    @ApiParam("工序代码")
    private String processCode;

    @ApiParam("工序名称")
    private String processName;

    @ApiParam("工艺代码")
    private String craftCode;

    @ApiParam("工艺代码")
    private String craftName;

    @ApiParam("当前工位设备对象([Json]MesCellEquipContext)")
    private String curCellEquipJson;

    @ApiParam("变量接口逻辑信息([Json]List<MesEquipmentVariableCfg>)")
    private String equipVariableCfgJson;

    @ApiParam("设备变量集合信息([Json]List<MesEquipmentVariable>)")
    private String equipVariableJson;

    @ApiParam("腔组明细信息")
    private String cavityGroupDetailJson;

    @ApiParam("腔组明细加工参数信息")
    private String cavityGroupProdParamJson;


    //-----------------根据业务执行情况必要时清理-----------------

    @ApiParam("加工结果:成功/报废/可疑 【报废/可疑: 在保存逻辑条码时有特殊业务处理】")
    private String productResult;

    @ApiParam("头道模具号")
    private String firstMouldNo;

    @ApiParam("模具号")
    private String mouldNo;

    @ApiParam("一模多腔信息")
    private String mouldCavityJson;

    @ApiParam("零件信息")
    private String partJson;

    @ApiParam("扫描信息:生产工单号")
    private String workOrderNoJson;

    @ApiParam("扫描信息:主条码")
    private String productSnJson;

    @ApiParam("扫描信息:装配件条码")
    private String assemblySnJson;

    @ApiParam("零件条码信息")
    private String produceSnDataJson;

    @ApiParam("生产工单信息")
    private String workOrderDataJson;

    @ApiParam("装配件信息(非排序/排序线排产装配目视项)")
    private String assemblyDataJson;

    @ApiParam("非排序加工规则信息")
    private String unSortRuleJson;

    @ApiParam("设备加工参数信息")
    private String equipProdParamJson;

    @ApiParam("时效性信息")
    private String timeEfficientJson;

    //--------------------------------------------------------

    public MesProductionProcessContext() {}

    public MesProductionProcessContext(String organizeCode) {
        this.organizeCode = organizeCode;
    }

    //上下文信息结果初始化
    public MesProductionProcessContext success() {
        this.success = true;
        this.message = null;
        return this;
    }

    //手动赋结果
    public MesProductionProcessContext success(Boolean success) {
        this.success = success;
        return this;
    }

    //上下文信息结果信息赋值后给工步使用
    public MesProductionProcessContext message(String message) {
        this.success = false;
        this.message = message;
        return this;
    }

    //从上下文中取出生产线对象
    public MesWorkCenter getWorkCenter() {
        return StringUtils.isEmpty(this.workCenterJson) ? null : JSONObject.parseObject(this.workCenterJson, MesWorkCenter.class);
    }

    //上下文赋值生产线对象
    public MesProductionProcessContext workCenterJson(MesWorkCenter workCenter) {
        this.workCenterJson = null != workCenter ? JSONObject.toJSONString(workCenter) : null;
        return this;
    }

    //从上下文中取出工位对象
    public MesWorkCell getWorkCell() {
        return StringUtils.isEmpty(this.workCellJson) ? null : JSONObject.parseObject(this.workCellJson, MesWorkCell.class);
    }

    //上下文赋值工位对象
    public MesProductionProcessContext workCellJson(MesWorkCell workCell) {
        this.workCellJson = null != workCell ? JSONObject.toJSONString(workCell) : null;
        return this;
    }

    //上下文赋值工序信息
    public MesProductionProcessContext processCode(String processCode, String processName) {
        this.processCode = processCode;
        this.processName = processName;
        return this;
    }

    //上下文赋值工艺信息
    public MesProductionProcessContext craftCode(String craftCode, String craftName) {
        this.craftCode = craftCode;
        this.craftName = craftName;
        return this;
    }

    //从上下文中取出工位设备对象集合
    private List<MesCellEquipContext> getCellEquipList() {
        return StringUtils.isEmpty(this.cellEquipListJson) ? null : JSONObject.parseArray(this.cellEquipListJson, MesCellEquipContext.class);
    }

    //上下文赋值工位设备对象集合
    public MesProductionProcessContext cellEquipListJson(List<MesCellEquipContext> cellEquipList) {
        this.cellEquipListJson = !CollectionUtils.isEmpty(cellEquipList) ? JSONObject.toJSONString(cellEquipList) : null;
        return this;
    }

    //从上下文中的工位设备对象集合中取出当前要使用的设备
    public MesCellEquipContext filterCurCellEquip() {
        List<MesCellEquipContext> cellEquipList = getCellEquipList();
        if (CollectionUtils.isEmpty(cellEquipList) || cellEquipList.size() == 1) return filterCurCellEquip(cellEquipList);
        return filterCurCellEquip(cellEquipList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getSeq()))).sorted(Comparator.comparing(MesCellEquipContext::getSeq)).collect(Collectors.toList()));
    }

    //从上下文中的工位设备对象集合中根据设备代码取出当前要使用的设备
    public MesCellEquipContext filterCurCellEquip(String equipmentCode) {
        if (StringUtils.isEmpty(equipmentCode)) return filterCurCellEquip();
        List<MesCellEquipContext> cellEquipList = getCellEquipList();
        return filterCurCellEquip(CollectionUtils.isEmpty(cellEquipList) ? null : cellEquipList.stream().filter(o -> (null != o && o.getEquipmentCode().equals(equipmentCode))).collect(Collectors.toList()));
    }

    //从上下文中的工位设备对象集合中取出第一个设备
    private MesCellEquipContext filterCurCellEquip(List<MesCellEquipContext> cellEquipContextList) {
        return CollectionUtils.isEmpty(cellEquipContextList) ? null : cellEquipContextList.get(0);
    }

    //从上下文中取出工位当前要使用的设备
    public MesCellEquipContext getCurCellEquip() {
        return StringUtils.isEmpty(this.curCellEquipJson) ? null : JSONObject.parseObject(this.curCellEquipJson, MesCellEquipContext.class);
    }

    //上下文赋值工位当前要使用的设备
    public MesProductionProcessContext curCellEquipJson(MesCellEquipContext curEquip) {
        this.curCellEquipJson = null != curEquip ? JSONObject.toJSONString(curEquip) : null;
        return this;
    }

    //从上下文中取出设备数据变量接口逻辑集合
    private List<MesEquipmentVariableCfg> getEquipVariableCfgList() {
        return StringUtils.isEmpty(this.equipVariableCfgJson) ? null : JSONObject.parseArray(this.equipVariableCfgJson, MesEquipmentVariableCfg.class);
    }

    //从上下文的设备数据变量接口逻辑对象集合中取出当前设备信息的逻辑类型对应的接口逻辑对象集合
    public List<MesEquipmentVariableCfg> getEquipVariableCfgListByVct() {
        MesCellEquipContext cellEquipContext = getCurCellEquip();
        return null == cellEquipContext ? null : getEquipVariableCfgListByVct(cellEquipContext.getVariableCfgType());
    }

    //从上下文的设备数据变量接口逻辑对象集合中取出当前设备信息的逻辑类型对应的接口逻辑对象集合
    public List<MesEquipmentVariableCfg> getEquipVariableCfgListByVct(String variableCfgType) {
        List<MesEquipmentVariableCfg> equipmentVariableCfgList = getEquipVariableCfgList();
        return (StringUtils.isEmpty(variableCfgType) || CollectionUtils.isEmpty(equipmentVariableCfgList)) ? null :
                equipmentVariableCfgList.stream().filter(o -> (null != o && o.getVariableCfgType().equals(variableCfgType))).collect(Collectors.toList());
    }

    //上下文赋值数据变量接口逻辑对象集合
    public MesProductionProcessContext equipVariableCfgJson(List<MesEquipmentVariableCfg> equipmentVariableCfgList) {
        if (CollectionUtils.isEmpty(equipmentVariableCfgList)) return this;
        if (!StringUtils.isEmpty(this.equipVariableCfgJson)) equipmentVariableCfgList.addAll(getEquipVariableCfgList());
        equipmentVariableCfgList = equipmentVariableCfgList.stream().filter(o -> null != o).sorted(Comparator.comparing(MesEquipmentVariableCfg::getModifyDatetime).reversed()).collect(Collectors.toList());
        equipmentVariableCfgList = equipmentVariableCfgList.stream().filter(o -> null != o).distinct().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(MesEquipmentVariableCfg::getId))), ArrayList::new));
        this.equipVariableCfgJson = JSONObject.toJSONString(equipmentVariableCfgList);
        return this;
    }

    //从上下文中取出设备数据变量对象集合
    private List<MesEquipmentVariable> getEquipVariableList() {
        return StringUtils.isEmpty(this.equipVariableJson) ? null : JSONObject.parseArray(this.equipVariableJson, MesEquipmentVariable.class);
    }

    //从上下文的设备数据变量对象集合中取出当前设备对应具体变量类型的数据变量对象集合
    public List<MesEquipmentVariable> getEquipVariableListByEquVt(Integer variableType) {
        MesCellEquipContext cellEquipContext = getCurCellEquip();
        return (null == cellEquipContext || StringUtils.isEmpty(variableType)) ? null : getEquipVariableListByEquVt(cellEquipContext.getEquipmentCode(), variableType);
    }

    //从上下文的设备数据变量对象集合中取出当前设备对应具体变量类型的数据变量对象集合
    public List<MesEquipmentVariable> getEquipVariableListByEquVt(String equipmentCode, Integer variableType) {
        List<MesEquipmentVariable> equipmentVariableList = getEquipVariableList();
        return (StringUtils.isEmpty(equipmentCode) || null == variableType || CollectionUtils.isEmpty(equipmentVariableList)) ? null :
                equipmentVariableList.stream().filter(o -> (null != o && o.getEquipmentCode().equals(equipmentCode) && o.getVariableType().compareTo(variableType) == 0)).collect(Collectors.toList());
    }

    //上下文赋值数据变量对象集合
    public MesProductionProcessContext equipVariableJson(List<MesEquipmentVariable> equipmentVariableList) {
        equipmentVariableList = CollectionUtils.isEmpty(equipmentVariableList) ? null : equipmentVariableList.stream().filter(o -> (null != o && !StringUtils.isEmpty(o.getCategoryLevelTwo()))).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(equipmentVariableList)) return this;
        if (!StringUtils.isEmpty(this.equipVariableJson)) equipmentVariableList.addAll(getEquipVariableList());
        equipmentVariableList = equipmentVariableList.stream().filter(o -> null != o).sorted(Comparator.comparing(MesEquipmentVariable::getModifyDatetime).reversed()).collect(Collectors.toList());
        equipmentVariableList = equipmentVariableList.stream().filter(o -> null != o).distinct().collect(Collectors.collectingAndThen(Collectors.toCollection(() -> new TreeSet<>(Comparator.comparing(MesEquipmentVariable::getId))), ArrayList::new));
        this.equipVariableJson = JSONObject.toJSONString(equipmentVariableList);
        return this;
    }

    //上下文赋值加工结果
    public MesProductionProcessContext productResult(String productResult) {
        this.equipVariableJson = productResult;
        return this;
    }

    //上下文清除加工结果
    public MesProductionProcessContext productResultClear() {
        this.equipVariableJson = null;
        return this;
    }

    //从上下文中取出头道模具号
    public MesEquipVariableCollectContext getFirstMouldNo() {
        return StringUtils.isEmpty(this.firstMouldNo) ? null : JSONObject.parseObject(this.firstMouldNo, MesEquipVariableCollectContext.class);
    }

    //上下文赋值头道模具号
    public MesProductionProcessContext firstMouldNo(MesEquipVariableCollectContext firstMouldNo) {
        this.firstMouldNo = null != firstMouldNo ? JSONObject.toJSONString(firstMouldNo) : null;
        return this;
    }

    //上下文清除头道模具号
    public MesProductionProcessContext firstMouldNoClear() {
        this.firstMouldNo = null;
        return this;
    }

    //从上下文中取出模具号
    public MesEquipVariableCollectContext getMouldNo() {
        return StringUtils.isEmpty(this.mouldNo) ? null : JSONObject.parseObject(this.mouldNo, MesEquipVariableCollectContext.class);
    }

    //上下文赋值模具号
    public MesProductionProcessContext mouldNo(MesEquipVariableCollectContext mouldNo) {
        this.mouldNo = null != mouldNo ? JSONObject.toJSONString(mouldNo) : null;
        return this;
    }

    //上下文清除模具号
    public MesProductionProcessContext mouldNoClear() {
        this.mouldNo = null;
        return this;
    }

    //从上下文中取出一模多腔信息集合
    public List<MesMouldMultiCavity> getMouldCavityList() {
        return StringUtils.isEmpty(this.mouldCavityJson) ? null : JSONObject.parseArray(this.mouldCavityJson, MesMouldMultiCavity.class);
    }

    //上下文赋值一模多腔信息
    public MesProductionProcessContext mouldCavityJson(List<MesMouldMultiCavity> mouldMultiCavityList) {
        this.mouldCavityJson = !CollectionUtils.isEmpty(mouldMultiCavityList) ? JSONObject.toJSONString(mouldMultiCavityList) : null;
        return this;
    }

    //上下文清除一模多腔信息
    public MesProductionProcessContext mouldCavityJsonClear() {
        this.mouldCavityJson = null;
        return this;
    }

    //从上下文中取出零件信息集合
    public List<MesPart> getPartList() {
        return StringUtils.isEmpty(this.partJson) ? null : JSONObject.parseArray(this.partJson, MesPart.class);
    }

    //上下文赋值零件信息集合
    public MesProductionProcessContext partJson(List<MesPart> partList) {
        this.partJson = !CollectionUtils.isEmpty(partList) ? JSONObject.toJSONString(partList) : null;
        return this;
    }

    //上下文清除零件信息
    public MesProductionProcessContext partJsonClear() {
        this.partJson = null;
        return this;
    }

    //从上下文中取出主条码信息
    public List<MesEquipVariableCollectContext> getProductSnList() {
        return StringUtils.isEmpty(this.productSnJson) ? null : JSONObject.parseArray(this.productSnJson, MesEquipVariableCollectContext.class);
    }

    //上下文赋值主条码信息
    public MesProductionProcessContext productSnJson(List<MesEquipVariableCollectContext> productSnList) {
        this.productSnJson = !CollectionUtils.isEmpty(productSnList) ? JSONObject.toJSONString(productSnList) : null;
        return this;
    }

    //上下文清除主条码信息
    public MesProductionProcessContext productSnJsonClear() {
        this.productSnJson = null;
        return this;
    }

    //从上下文中取出装配件条码集合
    public List<MesEquipVariableCollectContext> getAssemblySnList() {
        return StringUtils.isEmpty(this.assemblySnJson) ? null : JSONObject.parseArray(this.assemblySnJson, MesEquipVariableCollectContext.class);
    }

    //上下文赋值装配件条码集合
    public MesProductionProcessContext assemblySnJson(List<MesEquipVariableCollectContext> assemblySnList) {
        this.assemblySnJson = !CollectionUtils.isEmpty(assemblySnList) ? JSONObject.toJSONString(assemblySnList) : null;
        return this;
    }

    //上下文清除装配件条码集合
    public MesProductionProcessContext assemblySnJsonClear() {
        this.assemblySnJson = null;
        return this;
    }

    //从上下文中取出零件条码信息集合
    public List<MesProduceSn> getProduceSnDataList() {
        return StringUtils.isEmpty(this.produceSnDataJson) ? null : JSONObject.parseArray(this.produceSnDataJson, MesProduceSn.class);
    }

    //上下文赋值零件条码信息集合
    public MesProductionProcessContext produceSnDataJson(List<MesProduceSn> produceSnDataList) {
        this.produceSnDataJson = !CollectionUtils.isEmpty(produceSnDataList) ? JSONObject.toJSONString(produceSnDataList) : null;
        return this;
    }

    //上下文清除零件条码信息
    public MesProductionProcessContext produceSnDataJsonClear() {
        this.produceSnDataJson = null;
        return this;
    }

}
